@@ -14,19 +14,30 @@
 #' provided and the \code{x} aesthetic is set in the main plot.
 #' @param y The name of the variable along the y axis. Optional if \code{p} is
 #' provided and the \code{y} aesthetic is set in the main plot.
-#' @param type What type of marginal plot to show. One of: [density, histogram].
-#' @param margins Along Which margins to show the plots. One of: [both, x, y].
+#' @param type What type of marginal plot to show. One of: [density, histogram, boxplot].
+#' @param margins Along which margins to show the plots. One of: [both, x, y].
 #' @param size Integer describing the relative size of the marginal plots
 #' compared to the main plot. A size of 5 means that the main plot is 5x wider
 #' and 5x taller than the marginal plots.
-#' @param marginCol The colour to use for the outline of the marginal 
-#' density/histogram.
-#' @param marginFill The colour to use for the fill of the marginal histogram
-#' (not used when \code{type} is "density")
+#' @param ... Extra parameters to pass to the marginal plots. Any parameter that
+#' \code{geom_line()}, \code{geom_bar()}, or \code{geom_boxplot()} accept
+#' can be used. For example, \code{colour = "red"} can be used for any marginal plot type,
+#' and \code{binwidth = 10} can be used for histograms.
+#' @param xparams List of extra parameters to use only for the marginal plot along
+#' the x axis.
+#' @param yparams List of extra parameters to use only for the marginal plot along
+#' the y axis.
 #' @return An object of class ggExtraPlot. This extra class gets added onto
 #' a ggplot2 object in order for the \code{print} generic to easily work with
 #' this object. This means that the return value from this function can be
 #' printed or saved for later.
+#' @note Since the \code{size} parameter is used by \code{ggMarginal}, if you want
+#' to pass a size to the marginal plots, you cannot
+#' use the \code{...} parameter. Instead, you must pass \code{size} to
+#' both \code{xparams} and \code{yparams}. For example,
+#' \code{ggMarginal(p, size = 2)} will change the size of the main vs marginal plot,
+#' while \code{ggMarginal(p, xparams = list(size=2), yparams = list(size=2))}
+#' will make the density plot outline thicker.
 #' @examples
 #' if (requireNamespace("ggplot2", quietly = TRUE)) {
 #'   if (requireNamespace("gridExtra", quietly = TRUE)) {
@@ -41,6 +52,8 @@
 #'       ggMarginal(p2, type = "histogram")
 #'       ggMarginal(p2, margins = "x")
 #'       ggMarginal(p2, size = 2)
+#'       ggMarginal(p2, colour = "red")
+#'       ggMarginal(p2, colour = "red", xparams = list(colour = "blue", size = 3))
 #'       p2 <- p2 + ggplot2::ggtitle("Random data") + ggplot2::theme_bw(30)
 #'       ggMarginal(p2)
 #'       
@@ -54,9 +67,11 @@
 #'     }
 #'   }
 #' }
+#' @seealso \href{http://daattali.com/shiny/ggExtra-ggMarginal-demo/}{Demo Shiny app}
 #' @export
-ggMarginal <- function(p, data, x, y, type = "density", margins = "both",
-                       size = 5, marginCol = "black", marginFill = "grey") {
+ggMarginal <- function(p, data, x, y, type = c("density", "histogram", "boxplot"),
+                       margins = c("both", "x", "y"), size = 5,
+                       ..., xparams, yparams) {
 
   # Make sure the required packages are installed
   reqs <- c("grid", "gridExtra")
@@ -69,6 +84,29 @@ ggMarginal <- function(p, data, x, y, type = "density", margins = "both",
     })
   )  
   
+  # figure out all the default parameters
+  type <- match.arg(type)
+  margins <- match.arg(margins)
+  extraParams <- list(...)
+  if (is.null(extraParams[['colour']]) &&
+      is.null(extraParams[['color']]) &&
+      is.null(extraParams[['col']])) {
+    extraParams[['colour']] <- "black"
+  }
+  if (is.null(extraParams[['fill']])) {
+    extraParams[['fill']] <- "grey"
+  }
+  if (missing(xparams)) {
+    xparams <- list()
+  } else {
+    xparams <- as.list(xparams)
+  }
+  if (missing(yparams)) {
+    yparams <- list()
+  } else {
+    yparams <- as.list(yparams)
+  }
+  
   # Try to infer values for parameters that are missing from the input scatterplot
   if (missing(p)) {
     if (missing(data) | missing(x) | missing(y)) {
@@ -112,15 +150,49 @@ ggMarginal <- function(p, data, x, y, type = "density", margins = "both",
 
   textsize <- p$theme$text$size
 
-  if (type == "density") {
-    #marginPlot <- ggplot2::geom_density(fill = marginFill, col = marginCol)
-    marginPlot <- ggplot2::geom_line(stat = "density", col = marginCol)
-  } else if (type == "histogram") {
-    marginPlot <- ggplot2::geom_bar(fill = marginFill, col = marginCol)
-  } else {
-    stop(sprintf("`type` = `%s` is not supported", type), call. = FALSE)
-  }
-
+  # get the common code for both maginal (x and y) plots
+  marginPlot <- function(margin) {
+    if (margin == "x") {
+      if (type == "boxplot") {
+        plot <- ggplot2::ggplot(data, ggplot2::aes_string(x, x)) + ggplot2::coord_flip()
+      } else {
+        plot <- ggplot2::ggplot(data, ggplot2::aes_string(x))
+      }
+    } else if (margin == "y") {
+      if (type == "boxplot") {
+        plot <- ggplot2::ggplot(data, ggplot2::aes_string(y, y))
+      } else {
+        plot <- ggplot2::ggplot(data, ggplot2::aes_string(y)) + ggplot2::coord_flip()
+      }
+    } else {
+      stop(sprintf("`margin` = `%s` is not supported", margin), call. = FALSE)
+    }
+    
+    # add custom parameters specific to each marginal plot
+    # merge the parameters in an order that ensures that
+    # marginal plot params overwrite general params
+    if (margin == "x") {
+      extraParams <- append(xparams, extraParams)
+      extraParams <- extraParams[!duplicated(names(extraParams))]
+    } else if (margin == "y") {
+      extraParams <- append(yparams, extraParams)
+      extraParams <- extraParams[!duplicated(names(extraParams))]
+    }
+    
+    if (type == "density") {
+      extraParams[['stat']] <- "density"
+      layer <- do.call(ggplot2::geom_line, extraParams)
+    } else if (type == "histogram") {
+      layer <- do.call(ggplot2::geom_bar, extraParams)
+    } else if (type == "boxplot") {
+      layer <- do.call(ggplot2::geom_boxplot, extraParams)
+    } else {
+      stop(sprintf("`type` = `%s` is not supported", type), call. = FALSE)
+    }
+    
+    plot + layer
+  }  
+  
   # Create the horizontal margin plot
   # In order to ensure the marginal plots line up nicely with the main plot,
   # several things are done:
@@ -134,8 +206,7 @@ ggMarginal <- function(p, data, x, y, type = "density", margins = "both",
   # - Use the same axis range as the main plot
   if (margins != "y") {
     top <-
-      ggplot2::ggplot(data, ggplot2::aes_string(x)) +
-      marginPlot +
+      marginPlot("x") + 
       ggplot2::theme(
         text = ggplot2::element_text(size = textsize, color = "transparent"),
         line = ggplot2::element_blank(),
@@ -145,14 +216,14 @@ ggMarginal <- function(p, data, x, y, type = "density", margins = "both",
         axis.text.x = ggplot2::element_blank(),
         plot.margin = grid::unit(c(0, 0, -1, 0), "lines")) +
       ggplot2::ylab(p$labels$y) +
-      ggplot2::scale_x_continuous(limits = pb$panel$x_scales[[1]]$range$range)
+      ggplot2::scale_x_continuous(limits = get_limits(pb, "x"))
     
     # Add the longest y axis label to the top plot and ensure it's at a y value
     # that is on the plot (this is why I build the top plot, to know the y values)
     pbTop <- ggplot2::ggplot_build(top)
     top <-
       top +
-      ggplot2::scale_y_continuous(breaks = mean(pbTop$panel$y_scales[[1]]$range$range),
+      ggplot2::scale_y_continuous(breaks = mean(get_limits(pbTop, "y")),
                                   labels = ylabel)
     
     # If we are showing a marginal plot above the main plot, then transfer the
@@ -171,9 +242,7 @@ ggMarginal <- function(p, data, x, y, type = "density", margins = "both",
   # Create the vertical margin plot
   if (margins != "x") {
     right <-
-      ggplot2::ggplot(data, ggplot2::aes_string(y)) +
-      ggplot2::coord_flip() +
-      marginPlot +
+      marginPlot("y") + 
       ggplot2::theme(
         text = ggplot2::element_text(size = textsize, color = "transparent"),
         line = ggplot2::element_blank(),
@@ -183,7 +252,7 @@ ggMarginal <- function(p, data, x, y, type = "density", margins = "both",
         axis.text.y = ggplot2::element_blank(),
         plot.margin = grid::unit(c(0, 0, 0, -1), "lines")) +
       ggplot2::ylab(p$labels$x) +
-      ggplot2::scale_x_continuous(limits = pb$panel$y_scales[[1]]$range$range) +
+      ggplot2::scale_x_continuous(limits = get_limits(pb, "y")) +
       ggplot2::ggtitle(p$labels$title)
   }
 
@@ -242,3 +311,22 @@ print.ggExtraPlot <- function(x, ...) {
   grid::grid.newpage()
   grid::grid.draw(x)
 }
+
+# Get the axis range of the x or y axis of the given ggplot build object
+# This is needed so that if the range of the plot is manually changed, the
+# marginal plots will use the same range
+get_limits <- function(pb, axis) {
+  if (axis == "x") {
+    scales <- pb$panel$x_scales[[1]]
+  } else if (axis == "y") {
+    scales <- pb$panel$y_scales[[1]]
+  } else {
+    stop("Invalid `axis` parameter (only x and y are supported)", call. = FALSE)
+  }
+  
+  range <- scales$limits
+  if (is.null(range)) {
+    range <- scales$range$range
+  }
+  range
+}
\ No newline at end of file
